import template from "@babel/template";
import { declare } from "@babel/helper-plugin-utils";
const FUNCS_PATH = "cua.functions"; // TODO: configurable relative path
const CLIENT_PKG = "@cua.run/client";
let importedFuncs = [];
let clientImported = false;
let program;
export default declare((api) => {
    return {
        name: "cua.run",
        visitor: {
            ImportDeclaration: (path) => {
                if (path.node.importKind !== "value") {
                    return;
                }
                if (path.node.source.value === CLIENT_PKG) {
                    clientImported = true;
                }
                else if (path.node.source.value.match(FUNCS_PATH)) {
                    importedFuncs.push(...path.node.specifiers
                        .map((sp) => sp.type === "ImportDefaultSpecifier" &&
                        (sp.local.name || sp.local))
                        .filter((v) => !!v));
                    path.remove();
                }
            },
            Program(path) {
                program = path;
                clientImported = false;
                importedFuncs = [];
            },
            CallExpression(path) {
                if (path.node.callee.type !== "Identifier" ||
                    !importedFuncs.includes(path.node.callee.name)) {
                    return;
                }
                if (!clientImported) {
                    const importDeclaration = template(`
            import %%importName%% from %%importPath%%
          `);
                    program.unshiftContainer("body", importDeclaration({
                        importName: api.types.identifier("cua"),
                        importPath: api.types.stringLiteral("@cua.run/client"),
                    }));
                    clientImported = true;
                }
                const pushDeclaration = template.statement(`%%cexpr%%`);
                path.replaceWith(pushDeclaration({
                    cexpr: api.types.awaitExpression(api.types.callExpression(api.types.identifier(`cua.push`), [api.types.stringLiteral(path.node.callee.name), ...path.node.arguments])),
                }));
            },
        },
    };
});
